package de.unikiel.ipn.htmlform;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;

/**
 * A custom JDialog that asks for a multi line String input.
 * It also has a
 * button for
 * directly
 * parsing text from
 * a pdf file.
 * The owner is always the MainGUI (a singleton). 
 */
class AutoImportDialog extends JDialog implements MessageHandler {
    private static String value = null;
    private JTextArea textarea;
    private ProgressDialog progress;
    private static AutoImportDialog INSTANCE = new AutoImportDialog();

    // ActionListener for "Ok" and "Cancel" button
    private ActionListener returnValue = new ActionListener() {
        public void actionPerformed(ActionEvent e) {
            if ("Ok".equals(e.getActionCommand())) {
                value = textarea.getText();
            }
            else {
                value = null;
            }
            setVisible(false);
        }
    };
    
    private AutoImportDialog() {
        super(MainGUI.getInstance(), "auto import", true);
        
        //Create and initialize the buttons.
        JButton cancelButton = new JButton("Cancel");
        cancelButton.addActionListener(returnValue);
        //
        JButton okButton = new JButton("Ok");
        okButton.addActionListener(returnValue);
        //
        JButton pdfButton = new JButton("import from pdf");
        pdfButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                java.io.File file = getFile(INSTANCE);
                if (file == null) return;
                progress = new ProgressDialog(INSTANCE, INSTANCE);
                progress.showProgress("extracting from pdf");
                PDFParser parser = new PDFParser(file, progress);
                new Thread(parser).start();
                setEnabled(false);
            }
        });

        textarea  = new JTextArea();
        JScrollPane textScroller = new JScrollPane(textarea);

        JPanel textPane = new JPanel();
        textPane.setLayout(new BoxLayout(textPane, BoxLayout.PAGE_AXIS));
        JLabel label = new JLabel(
            "copy text to parse from into the text field " 
            + "or choose \"import from pdf\""
        );
        label.setLabelFor(textarea);
        textPane.add(label);
        textPane.add(Box.createRigidArea(new Dimension(0,5)));
        textPane.add(textScroller);
        textPane.setBorder(BorderFactory.createEmptyBorder(10,10,10,10));

        JPanel buttonPane = new JPanel();
        buttonPane.setLayout(new BoxLayout(buttonPane, BoxLayout.LINE_AXIS));
        buttonPane.setBorder(BorderFactory.createEmptyBorder(0, 10, 10, 10));
        buttonPane.add(Box.createHorizontalGlue());
        buttonPane.add(cancelButton);
        buttonPane.add(Box.createRigidArea(new Dimension(10, 0)));
        buttonPane.add(pdfButton);
        buttonPane.add(Box.createRigidArea(new Dimension(10, 0)));
        buttonPane.add(okButton);

        Container contentPane = getContentPane();
        contentPane.add(textPane, BorderLayout.CENTER);
        contentPane.add(buttonPane, BorderLayout.PAGE_END);
        
        pack();
    }
    
    /**
     * Returns always the same instance of this singleton.
     * This Dialog is modal by default. 
     * @return always the same instance of AutoImportDialog
     */
    static AutoImportDialog getInstance() {
        return INSTANCE;
    }

    /**
     * Shows the dialog.
     * Returns when this dialog is closed.
     * The user can do that by pressing ok, cancel or close.
     * @return the content of the text area, null
     * if no text has been entered or the user pressed
     * cancel or close 
     */ 
    String showDialog() {
        setSize(600, 400);
        setVisible(true);
        return value;
    }

    /**
     * Handles a Message.
     * Messages of type Message.RESULT_MESSAGE are displayed
     * on the text area. All other Messages are displayed in
     * a seperate modal Dialog with this Dialog as the owner.
     * If this Dialog is disabled then it is enabled in the
     * event dispatching thread.
     * @param msg the Message that shall be handled.
     * @throws NullPointerException if msg == null. 
     */
    public void handleMessage(Message msg) {
        switch (msg.getType()) {
            case Message.RESULT_MESSAGE : 
                textarea.setText(msg.getMessage());
                break;
            default:
                MainGUI.getInstance().print(this, msg);
        }
        if (!isEnabled()) { // equivalent: progressDialog != null
            SwingUtilities.invokeLater(new Runnable() { 
                public void run() {
                    progress.dispose();
                    progress = null;
                    setEnabled(true);
                    show();
                }
            });
        }
    }

    // ask for a pdf file
    private java.io.File getFile(Component frameComp) {
        JFileChooser chooser = new JFileChooser();
        chooser.setFileFilter(new javax.swing.filechooser.FileFilter() {
            public boolean accept(File f) {
                if (f.isDirectory() || f.getName().matches(".*\\.pdf")) {
                    return true;
                }
                return false;
            }
            
            public String getDescription() {
                return "pdf-files (*.pdf)";
            }
        });
        int ret = chooser.showOpenDialog(frameComp);
        if (ret != JFileChooser.APPROVE_OPTION) {
            return null;
        }
        return chooser.getSelectedFile();
    }

}

