package de.unikiel.ipn.htmlform;
import java.util.*;
import java.io.*;

/**
 * A html converter.
 */
class HTMLVisitor implements TextVisitor {
    private Properties properties;
    private MessageHandler successor;
    
    /**
     * Creates a new instance of this HTMLVisitor.
     * @param handler the MessageHandler for error messages
     * that may occur.
     */
    HTMLVisitor(MessageHandler handler) {
        successor = handler;
        try {
            properties = new Properties();
            InputStream in = getClass().getResourceAsStream("htmlencoding.ini");
            if (in == null) {
                handleMessage(new Message(
                    "Error loading properties for html encoding.\n"
                    + "Reason: somebody messed up the executable jar file!\n"
                    + "I will continue anyway, but all special characters\n"
                    + "may be displayed in unicode syntax.",
                    Message.ERROR_MESSAGE));
            }
            else {
                properties.load(in);
                in.close();
            }
        } catch (IOException e) {
            handleMessage(new Message(
                "Error loading properties for html encoding.\n"
                + "I will continue anyway, but all special characters\n"
                + "may be displayed in unicode syntax.",
                Message.ERROR_MESSAGE));
        }
    }

    /**
     * Converts all htmlentities in the entry's content.
     * @param e the Entry. 
     */
    public void visit(Entry e) {
        StringBuffer text = new StringBuffer(e.getContent());
        for (int i = 0; i < text.length(); i++) {
            char c = text.charAt(i);
            /* normal letter or space? */
            if (c < 'A' || (c > 'Z' && (c < 'a' || c > 'z'))) {
                String replace = getUnicodeString(c);
                text.replace(i, i+1, replace);
                i += replace.length() - 1;
            }
        }
        e.changeContent(text.toString());
    }

    private String getUnicodeString(char c) {
        String ret = "" + Integer.toHexString(c);
        while (ret.length() < 4) {
            ret = "0" + ret;
        }

        return properties.getProperty(ret, "&#x" + ret + ";");
    }

    /**
     * Forwards the Message to the successor.
     * See constructor.
     * @param msg the message that shall be passed to the successor. 
     */
    public final void handleMessage(Message msg) {
        successor.handleMessage(msg);
    }
}
