package de.unikiel.ipn.htmlform;
import java.awt.event.*;
import java.io.*;
import javax.swing.*;
import java.util.regex.*;

/**
 * Manages user input.
 */
public class InputController implements WindowListener, ActionListener,
        KeyListener {
    private static InputController INSTANCE = new InputController();
    
    /**
     * Returns the instance of this singleton. 
     * @return always the same instance of an InputController
     */
    public static InputController getInstance() {
        return INSTANCE;
    }

    /**
     * Search for suspicious words that contain 
     * a mini-space in the middle.
     */
    public void keyPressed(KeyEvent e) {
        if (!(e.getKeyCode() == KeyEvent.VK_F3) 
                || !(e.getComponent() instanceof JTextArea)) {
            return;
        }
        JTextArea source = (JTextArea) e.getComponent();
        Pattern pattern;
        pattern = Pattern.compile(
                    "(i \\w)|(d \\w)|(l \\w)|(n \\w)|(f \\w)|(o \\w)");
        Matcher m = pattern.matcher(source.getText());
        if (m.find(source.getCaretPosition())) {
            source.setSelectionStart(m.start() + 1);
            source.setSelectionEnd(m.start() + 2);
        }
    }
    
    public void actionPerformed(ActionEvent e) {
        if (e.getActionCommand().equals("start")) {
            ZFDNModel.getInstance().update();
            return;
        }
        else if (e.getActionCommand().equals("parse")) {
            ZFDNModel.getInstance().parse();
            return;
        }
        else if (e.getActionCommand().equals("open")) {
            File file = ZFDNModel.getInstance().askForProjectFile();
            if (file == null) {
                return;
            }
            ZFDNModel.getInstance().readProjectFile(file);
            ZFDNModel.getInstance().createEntryPanels();
            MainGUI.getInstance().arrangeComponents(
                ZFDNModel.getInstance().getComponents()
            );
            return;
        }
        else {
            assert (false) : 
                "unrecognized action command: " + e.getActionCommand();
        }
    }

    
    public void focusGained(FocusEvent e) {
        if (!(e.getSource() instanceof JTextArea)) {
            return;
        }
        ((JTextArea) e.getSource()).selectAll();
    }


    /**
     * Calls shutdown().
     * @see #shutdown() 
     */
    public void windowClosing(WindowEvent e) {
        shutdown();
    }

    /**
     * Exits the program.
     */
    public void shutdown() {
        MainGUI.getInstance().setVisible(false);
        MainGUI.getInstance().dispose();
        System.exit(0);
    }
    
    public void windowClosed(WindowEvent event) {}
    public void windowDeactivated(WindowEvent event) {}
    public void windowDeiconified(WindowEvent event) {}
    public void windowIconified(WindowEvent event) {}
    public void windowOpened(WindowEvent event) {}
    public void windowActivated(WindowEvent event) {}
    public void focusLost(FocusEvent e) {}
    public void keyTyped(KeyEvent e) {}
    public void keyReleased(KeyEvent e) {}
}

