package de.unikiel.ipn.htmlform;
import java.awt.Component;
import javax.swing.JOptionPane;

/**
 * A Message.
 * @see MessageHandler
 */
public class Message {
    /**
     * An error message.
     * Error messages are usually displayed in a JDialog by the
     * MessageHandler that has the focus (a <tt>java.awt.Window</tt>).
     */
    public static final int ERROR_MESSAGE = JOptionPane.ERROR_MESSAGE;
    /**
     * An information message.
     * Information messages are usually displayed in a JDialog by the
     * MessageHandler that has the focus (a <tt>java.awt.Window</tt>).
     */
    public static final int INFORMATION_MESSAGE = 
        JOptionPane.INFORMATION_MESSAGE;
    /**
     * A result of a calculation.
     * It is usually applied by the <tt>MessageHandler</tt> that 
     * has waited for a result. How it is used depends on the
     * <tt>MessageHandler</tt>
     */
    public static final int RESULT_MESSAGE = -2727;
    private String message;
    private int type;
    
    /**
     * Create a new Message.
     * @param message the message text
     * @param type the message type
     * @see Message#ERROR_MESSAGE
     * @see Message#INFORMATION_MESSAGE
     * @see Message#RESULT_MESSAGE
     */
    public Message(String message, int type) {
        this.message = message;
        this.type = type;
    }
    
    /**
     * Returns the message text.
     * @return the message text
     */
    public String getMessage() {
        return message;
    }
    
    /**
     * Returns the message type.
     * @return the message type
     * @see Message#ERROR_MESSAGE
     * @see Message#INFORMATION_MESSAGE
     * @see Message#RESULT_MESSAGE
     */
    public int getType() {
        return type;
    }

    /**
     * Returns the message text.
     * @return the message text
     * @see Message#getMessage()
     */
    public String toString() {
        return getMessage();
    }
}

