package de.unikiel.ipn.htmlform;

/**
 * An interface to handle Messages.
 * The intended use is to provide a chain of responsibility
 * for handling a message. Each receiver of a message knows
 * its successor in the chain. It may pass the message to the
 * successor or handle it by itself.
 * @see Message
 */
interface MessageHandler {
    /**
     * Handles the Message or forwards it to the successor.
     * If it handles it by itself may depend for example on the
     * type of the Message or on wether it knows which <tt>java.awt.Window</tt>
     * is currently focused. 
     * @param msg the Message that has to be handled.
     */
    public void handleMessage(Message msg);
}

