package de.unikiel.ipn.htmlform;
import java.io.*;

/**
 * A class to perform updates on the target files.
 * @see Entry 
 */
class Updater implements MessageHandler {
    private TextVisitor[] visitors;
    private Entry[] entries;
    private MessageHandler successor;

    /**
     * Creates a new Updater instance.
     * @param e an array containing the entries that contain
     *          the information which files shall be updated and how.
     * @param handler the message handler that shall handle the messages
     *                that occur during the update progress. It should be
     *                the currently focused Frame or Dialog.
     * @throws NullPointerException when one of the arguments is null. 
     */
    Updater(Entry[] e, MessageHandler handler) {
        if (e == null || handler == null) {
            throw new NullPointerException(
                "Arguments must not be null"
            );
        }
        visitors = new TextVisitor[] {
            new AntiLineBreakVisitor(this),
            new HTMLVisitor(this)
        };
        entries = e;
        successor = handler;
    }
    
    /**
     * Performs the final updates on the target files based on
     * the information in the Entries.
     * @see Entry 
     */
    void update() {
        /* visit all texts (create html-code) */
        for (int e = 0; e < entries.length; e++) {
            for (int v = 0; v < visitors.length; v++) {
                entries[e].accept(visitors[v]);
            }
        }
        new SimpleAppender(this).append(entries);
    }

    public void handleMessage(Message msg) {
        successor.handleMessage(msg);
    }
}
 
