import java.util.regex.*;
import java.util.Date;
import java.text.SimpleDateFormat;

/**
 * static utility methods for parsing parts of the document<br>
 * all methods throw a NullPointerException when an argument is null!
 */
public class AutoParser {
    private static final String pageNumbers = 
        "Jg\\. \\d\\, \\d\\d\\d\\d\\, S\\. \\d+ ?- ?\\d+";
    private static final String author = "[A-Z &\\.\\,-]{8,100}";
    private static final Pattern ABSTRACT_PATTERN = 
        Pattern.compile("(Summary:?\\n)|(Abstract:?\\n)");
    private static final Pattern ZUSAMMENFASSUNG_PATTERN =
        Pattern.compile("(Zusammenfassung:?\\n)");
    /* things to keep lowercase */
    private final String LOW = "((UND)|(VON)|(VAN))\\W.*";        

    public AutoParser() {}
    
    public String parseEntry1(String source) {
        String ret = parseAuthor(source);
        if (ret == null) {
            return null;
        }
        else {
            return toLowCase(ret);
        }
    }
    
    public String parseEntry2(String source) {
        return parseTitle(source);
    }
    
    public String parseEntry3(String source) {
        return parseArticleCode(source);
    }
    
    // Date
    public String parseEntry4(String source) {
        return null;
    }
    
    public String parseEntry5(String source) {
        return "(Stand: " + 
               new SimpleDateFormat("dd.MM.yyyy").format(new Date()) + ")";
    }
    
    public String parseEntry6(String source) {
        return parseAuthor(source);
    }
    
    public String parseEntry7(String source) {
        return parseArticleCode(source);
    }
    
    public String parseEntry8(String source) {
        return parseTitle(source);
    }

    public String parseEntry9(String source) {
        return parseArticleCode(source);
    }
    
    public String parseEntry10(String source) {
        return parsePageNumbers(source);
    }
    
    public String parseEntry11(String source) {
        return parseEntry1(source);
    }
    
    public String parseEntry12(String source) {
        return parseTitle(source);
    }
    
    // Dateiname
    public String parseEntry13(String source) {
        int index1 = source.indexOf("FILENAME:") + "FILENAME:".length();
        int index2 = source.indexOf("\nFILESIZE:");
        if (index1 >= index2 || index2 == -1) {
            return null;
        }
        return source.substring(index1, index2);
    }
    
    // Dateigre
    public String parseEntry14(String source) {
        int index = source.indexOf("FILESIZE:") + "FILESIZE:".length();
        if (index >= source.length()) {
            return null;
        }
        try {
            return ""
                   + Math.round(
                        (Integer.parseInt(source.substring(index)) / 1024))
                   + " K";
        } catch (NumberFormatException e) {
            return null;
        }
    }
    
    public String parseEntry15(String source) {
        return parseZusammenfassung(source);
    }
    
    public String parseEntry16(String source) {
        return parseAbstract(source);
    }
    
    public String parseEntry17(String source) {
        return null;
//        return 
//            "<!-- #BeginDate format:Ge1 -->"
//            + new SimpleDateFormat("dd.MM.yyyy").format(new Date())
//            + "<!-- #EndDate -->";
    }
    
    public String parseEntry18(String source) {
        return null;
        //return parseEntry17(source);
    }
    
    private String toLowCase(String s) {
        StringBuffer text = new StringBuffer(s);
        for (int i=0; i < text.length(); i++) {
            // set all non-word-start to lowercase
            if (i != 0 && (
                            !(Character.isWhitespace(text.charAt(i - 1))
                                || text.charAt(i - 1) == '-')
                            )) {
                text.setCharAt(i, Character.toLowerCase(text.charAt(i)));
            }
            // set all non-names to lowercase
            if (text.substring(i).matches(LOW)) {
                 if (i != 0) {
                    if (Character.isWhitespace(text.charAt(i - 1))
                                || text.charAt(i - 1) == '-') {
                        text.setCharAt(i, 
                            Character.toLowerCase(text.charAt(i)));
                    }
                }
            }
        }
        return text.toString();
    }
    
    private static String parsePageNumbers(String source) {
        return parse(source, pageNumbers);
    }
    
    private static String parseAuthor(String source) {
        return parse(source, author);
    }
    
    private String parseTitle(String source) {
        String _author = parse(source, author);
        if (_author == null) {
            return "could not parse title - author missing as high delimiter";
        }
        int index1 = source.indexOf(_author) + _author.length();
        
        int index2 = source.indexOf("Zusammenfassung");
        int index2_alt = source.indexOf("\n\n", index1 + 5);
        if (index2_alt < index2) {
            index2 = index2_alt;
        }
        
        if (index2 < index1) {
            return "could not parse title - "
                   + "no Zusammenfassung found as lower delimiter";
        }
        return source.substring(index1, index2).replace('\n', ' ').trim();
    }

    private static String parseZusammenfassung(String source) {
        // find above delimiter for Zusammenfassung
        Matcher m = ZUSAMMENFASSUNG_PATTERN.matcher(source);
        if (!m.find()) {
            return "Zusammenfassung not found - enter manually";
        }
        int index1 = m.start() + m.group().length();
        
        //find lower delimiter for Zusammenfassung (start of abstract / summary)
        int index2 = source.indexOf(parseAbstract(source));
        
        // substract header of lower delimiter
        m = ABSTRACT_PATTERN.matcher(source);
        m.find();
        index2 -= m.group().length();

        if (index2 < index1) {
            return "Abstract missing as lower delimiter for Zusammenfassung!";
        }
        return source.substring(index1, index2);
    }
    
    private static String parseAbstract(String source) {
        Matcher m = ABSTRACT_PATTERN.matcher(source);
        if (!m.find()) {
            return "no abstract / summary found - enter manually";
        }
        int index1 = m.start() + m.group().length();

        int index2 = source.indexOf("\nZeitschrift fr");
        int index2_alt = source.indexOf("\n\n", index1 + 5);
        if (index2_alt < index2) {
            index2 = index2_alt;
        }

        if (index2 == -1 || index2 < index1) {
            index2 = source.length();
        }
        return source.substring(index1, index2);
    }
    
    private String parseArticleCode(String source) {
        // get filename first
        String s1 = parseEntry13(source);
        if (s1 == null) {
            return "could not parse article code - enter manually";
        }
        // split off a part of it
        String[] g = s1.split("_");
        if (g.length < 1) {
            return s1;
        }
        return "Art" + g[0];
    }
    
    private static String parse(String source, String pattern) {
        Pattern p = Pattern.compile(pattern);
        Matcher m = p.matcher(source);
        if (!m.find()) return null;
        return m.group();
    }
}

