import java.util.regex.*;
import java.util.Date;
import java.text.SimpleDateFormat;

/**
 * static utility methods for parsing parts of the document<br>
 * all methods throw a NullPointerException when an argument is null!
 */
public class BerichteParser {
    private static final String pageNumbers = 
        "Jg\\. \\d\\, \\d\\d\\d\\d\\, S\\. \\d+ ?- ?\\d+";
    private static final String author = 
        "[A-Z &\\.\\,-]{15,100}\\n?([A-Z &\\.\\,-]{3,100})?";
    private static final Pattern ABSTRACT_PATTERN = 
        Pattern.compile("(Summary:?\\n)|(Abstract:?\\n)");
    private static final Pattern ZUSAMMENFASSUNG_PATTERN =
        Pattern.compile("(Zusammenfassung:?\\n)");
    /* things to keep lowercase */
    private final String LOW = "((UND)|(VON)|(VAN))\\W.*";        

    public BerichteParser() {}
    
    public String parseEntry1(String source) {
        String ret = parseAuthor(source);
        if (ret == null) {
            return null;
        }
        else {
            return toLowCase(ret);
        }
    }
    
    public String parseEntry2(String source) {
        return parseTitle(source);
    }
    
    public String parseEntry3(String source) {
        return parseArticleCode(source);
    }
    
    // Date
    public String parseEntry4(String source) {
        return null;
    }
    
    public String parseEntry5(String source) {
        return "(Stand: " + 
               new SimpleDateFormat("dd.MM.yyyy").format(new Date()) + ")";
    }
    
    public String parseEntry6(String source) {
        return parseAuthor(source);
    }
    
    public String parseEntry7(String source) {
        return parseArticleCode(source);
    }
    
    public String parseEntry8(String source) {
        return parseTitle(source);
    }

    public String parseEntry9(String source) {
        return parseArticleCode(source);
    }
    
    public String parseEntry10(String source) {
        return parsePageNumbers(source);
    }
    
    public String parseEntry11(String source) {
        return parseEntry1(source);
    }
    
    public String parseEntry12(String source) {
        return parseTitle(source);
    }
    
    // Dateiname
    public String parseEntry13(String source) {
        int index1 = source.indexOf("FILENAME:") + "FILENAME:".length();
        int index2 = source.indexOf("\nFILESIZE:");
        if (index1 >= index2|| index2 == -1) {
            return null;
        }
        return source.substring(index1, index2);
    }
    
    // Dateigre
    public String parseEntry14(String source) {
        int index = source.indexOf("FILESIZE:") + "FILESIZE:".length();
        if (index >= source.length()) {
            return null;
        }
        try {
            return ""
                   + Math.round(
                        (Integer.parseInt(source.substring(index)) / 1024))
                   + " K";
        } catch (NumberFormatException e) {
            return null;
        }
    }
    
    public String parseEntry15(String source) {
        return "";
    }
    
    public String parseEntry16(String source) {
        return "";
    }
    
    public String parseEntry17(String source) {
        return null;
//        return 
//            "<!-- #BeginDate format:Ge1 -->"
//            + new SimpleDateFormat("dd.MM.yyyy").format(new Date())
//            + "<!-- #EndDate -->";
    }
    
    public String parseEntry18(String source) {
        return null;
        //return parseEntry17(source);
    }
    
    private String toLowCase(String s) {
        StringBuffer text = new StringBuffer(s);
        for (int i=0; i < text.length(); i++) {
            // set all non-word-start to lowercase
            if (i != 0 && (
                            !(Character.isWhitespace(text.charAt(i - 1))
                                || text.charAt(i - 1) == '-')
                            )) {
                text.setCharAt(i, Character.toLowerCase(text.charAt(i)));
            }
            // set all non-names to lowercase
            if (text.substring(i).matches(LOW)) {
                 if (i != 0) {
                    if (Character.isWhitespace(text.charAt(i - 1))
                                || text.charAt(i - 1) == '-') {
                        text.setCharAt(i, 
                            Character.toLowerCase(text.charAt(i)));
                    }
                }
            }
        }
        return text.toString();
    }
    
    private static String parsePageNumbers(String source) {
        return null;
    }
    
    private static String parseAuthor(String source) {
        return parse(source, author);
    }
    
    private String parseTitle(String source) {
        String startMark = "\n";
        String endMark = "\n\n";
        int index1 = source.indexOf(startMark);
        if (index1 == -1) {
            return "could not parse title - expected after two newlines";
        }

        int index2 = source.indexOf(endMark, index1 + startMark.length());
        if (index1 + startMark.length() >= index2) {
            return "could not parse title - expected two newlines after title";
        }
        
        return source.substring(index1 + startMark.length(), index2);
    }

    private static String parseZusammenfassung(String source) {
        return "";
    }
    
    private static String parseAbstract(String source) {
        return "";
    }
    
    private String parseArticleCode(String source) {
        String s1 = parseEntry13(source);
        if (s1 == null) {
            return "could not parse article code - enter manually";
        }
        String[] g = s1.split("\\.");
        if (g.length < 1) {
            return s1;
        }
        return g[0];
    }
    
    private static String parse(String source, String pattern) {
        Pattern p = Pattern.compile(pattern);
        Matcher m = p.matcher(source);
        if (!m.find()) return null;
        return m.group();
    }
}
